<?php

use ExceptionHandle\HttpNotFoundError;
use ExceptionHandle\PDOError;
use ExceptionHandle\TypeErrorParameters;
use Gateway\GatewayForm;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;

require 'Config/vendor/autoload.php';

/**
 * Instantiate App
 */
$app = AppFactory::create();

// Add Routing Middleware
$app->addRoutingMiddleware();

/**
 * Add Error Handling Middleware
 *
 * @param bool $displayErrorDetails -> Should be set to false in production
 * @param bool $logErrors -> Parameter is passed to the default ErrorHandler
 * @param bool $logErrorDetails -> Display error details in error log
 */
$errorMiddleware = $app->addErrorMiddleware(true, true, true);

/**
 * Add a route for the API
 */
$app->get('/', function (Request $request, Response $response) {
    throw new HttpNotFoundError($request);
});

$app->get('/getForm', function(Request $request, Response $response){
    try{
        $response->getBody()->write(json_encode((new GatewayForm)->getForm(), JSON_UNESCAPED_UNICODE));
    }catch (PDOException $e){
        throw new PDOError($request,$e->getMessage(),$e);
    }
    return $response->withHeader('Content-type', 'application/json')->withStatus(200);
});

$app->put('/insertForm', function(Request $request, Response $response){
    $parameters = $request->getQueryParams();
    if (empty($parameters['title']) || empty($parameters['description'])){
        throw new TypeErrorParameters($request);
    }
    try{
        (new GatewayForm)->insertForm($parameters['title'],$parameters['description']);
    }catch (PDOException $e){
        throw new PDOError($request,$e->getMessage(),$e);
    }
});

$app->put('/deleteForm', function(Request $request, Response $response){
    $parameters = $request->getQueryParams();
    if (empty($parameters['id'])){
        throw new TypeErrorParameters($request);
    }
    try{
        (new GatewayForm)->deleteForm($parameters['id']);
    }catch (PDOException $e){
        throw new PDOError($request,$e->getMessage(),$e);
    }
});

$app->get('/selectForDeleteAndInsert', function(Request $request, Response $response){
    $parameters = $request->getQueryParams();
    if (empty($parameters['id']) || empty($parameters['response'])){
        throw new TypeErrorParameters($request);
    }
    try{
        $response->getBody()->write(json_encode((new GatewayForm)->selectForDeleteAndInsert($parameters['id'],$parameters['response']), JSON_UNESCAPED_UNICODE));
    }catch (PDOException $e){
        throw new PDOError($request,$e->getMessage(),$e);
    }
    return $response->withHeader('Content-type', 'application/json')->withStatus(200);
});

$app->put('/assignKeywordToQuestion', function(Request $request, Response $response){
    $parameters = $request->getQueryParams();
    if (empty($parameters['keyword']) || empty($parameters['id']) || empty($parameters['response'])){
        throw new TypeErrorParameters($request);
    }
    try{
        (new GatewayForm)->assignKeywordToQuestion($parameters['keyword'],$parameters['id'],$parameters['response']);
    }catch (PDOException $e){
        throw new PDOError($request,$e->getMessage(),$e);
    }
});


// Run app
$app->run();
